/*
 *  Warning: This is a beerware modification of the BSD licence
 * 
 *	Copyright (c) 2007, Peter Kuhar http://www.pkuhar.com
 *	
 *	All rights reserved.
 *	
 *	Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions are met:
 *	
 *	    * Redistributions of source code must retain the above copyright notice,
 *        this list of conditions and the following disclaimer.
 *	    * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *	    * Neither the name of the Peter Kuhar nor the names of its contributors 
 *        may be used to endorse or promote products derived from this software 
 *        without specific prior written permission.
 *	    * Whoever uses this software is morally obliged to buy the author a beer 
 *        they happen to meet.
 *      * Whoever uses this software for comercial purposes is morally obliged to buy 
 *        the author a 6pack of beer for every 1000 earned directly(including selling
 *        a hardware that is supported by this software ) from this software.
 *	
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *	NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include <avr/io.h>
#include "usb.h"
#include "dbg_int.h"


void delay( unsigned char d )
{
	volatile unsigned char i=d;
	while( i-- );
}

void dbg_init()
{
	DBG_DDR = (DBG_DDR & ~DBG_MASK) | EXP2(PIN_DATA) | EXP2(PIN_CLOCK);//data and clock out, leave reset high( not in reset )
	DBG_PORT &= EXP2(PIN_RESET);//reset pin low, to emulate open collector
	
	SET(STATUS_LED);
	OUTPUT(STATUS_LED);
}

unsigned char dbg_send_byte( unsigned char ch, unsigned char dir )
{
	unsigned char cnt;
	if( dir )
		DBG_DIR_OUT();
	else
		DBG_DIR_IN();
		
	for( cnt=8; cnt; cnt-- ){
		DBG_DATA( ch & 0x80 );
		ch <<= 1;
		DBG_CLOCK_HIGH();
		
		delay_6();
		
		if( DBG_IN )
			ch |= 0x01;
			
		DBG_CLOCK_LOW();
		//delay_6();
	}
	return ch;
}

void dbg_command( unsigned char *pIn, unsigned char cInCnt, unsigned char *pOut, unsigned char cOutCnt )
{
	while( cInCnt--){
		dbg_send_byte( *pIn++, DIR_OUT );
	}
	while( cOutCnt--){
		*pOut++ = dbg_send_byte( 0x00, DIR_IN );
	}
}

void dbg_enter()
{
	DBG_RESET( 1 );
	delay(200);
	DBG_CLOCK_HIGH();
	delay(1);
	DBG_CLOCK_LOW();
	delay(1);
	DBG_CLOCK_HIGH();
	delay(1);
	DBG_CLOCK_LOW();
	delay(200);
	DBG_RESET( 0 );
}

#define PAR1_LOW	data[2]
#define PAR1_HIGH	data[3]
#define PAR2_LOW	data[4]
#define PAR2_HIGH	data[5]

#define READ_LEN	data[6]

extern	byte_t	usb_setup ( byte_t data[8] )
{
	byte_t	req;
	byte_t	read_len = READ_LEN;
	
	// Generic requests
	req = data[1];
	if	( req == VENDOR_LED ){
		if( PAR1_LOW ) {
			SET(STATUS_LED);
		}
		else {
			CLR(STATUS_LED);
		}
		return 0;
	}
	
	if	( req == VENDOR_RESET ){
		DBG_RESET( PAR1_LOW );
		return 0;
	}
	
	if	( req == VENDOR_ENTER_DEBUG_MODE ){
		dbg_enter();
		return 0;
	}
	
	if ( req == VENDOR_DEBUG ){
		dbg_command( &PAR1_LOW, (PAR1_LOW & 0x3) + 1, data, read_len);
		return read_len;
	}
	
	if ( req == VENDOR_READCODE ){
		return 0xFF;
	}
	
	return 0;
}


unsigned char tmp[3];
unsigned char dbg_read_byte;

void dbg_debuginst1( unsigned char par1)
{
	tmp[0] = DEBUG_INSTR1; tmp[1] = par1;
	dbg_command( tmp,2, &dbg_read_byte, 1);
}

void dbg_writeXDATA(unsigned char chData)
{
	//MOV A,chData
	tmp[0] = DEBUG_INSTR2; tmp[1] = 0x74; tmp[2] = chData;
	dbg_command( tmp,3, &chData, 1);
	
	//MOVX @DPTR, A
	dbg_debuginst1( 0xF0 ); 
	//INC DPTR
	dbg_debuginst1( 0xA3 ); 
}

unsigned char readcode_temp;
void dbg_readcode()
{
	//CLR A
	dbg_debuginst1(0xE4);
	
	//MOVC A, @A+DPTR
	dbg_debuginst1( 0x93);
	readcode_temp = dbg_read_byte;
		
	//INC DPTR
	dbg_debuginst1(0xA3);
}

// ----------------------------------------------------------------------
// Handle an OUT packet.
// ----------------------------------------------------------------------
extern	void	usb_out ( byte_t* data, byte_t len )
{
	while( len-- ){
		dbg_writeXDATA( *data++ );
	}
}

// ----------------------------------------------------------------------
// Handle an IN packet.
// ----------------------------------------------------------------------
extern	byte_t	usb_in ( byte_t* data, byte_t len )
{
	unsigned char cnt = len;
	while( cnt-- ){
		dbg_readcode();
		*data++ = readcode_temp;
	}
	return len;
}
