/*
 *  Warning: This is a beerware modification of the BSD licence
 * 
 *	Copyright (c) 2007, Peter Kuhar http://www.pkuhar.com
 *	
 *	All rights reserved.
 *	
 *	Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions are met:
 *	
 *	    * Redistributions of source code must retain the above copyright notice,
 *        this list of conditions and the following disclaimer.
 *	    * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *	    * Neither the name of the Peter Kuhar nor the names of its contributors 
 *        may be used to endorse or promote products derived from this software 
 *        without specific prior written permission.
 *	    * Whoever uses this software is morally obliged to buy the author a beer 
 *        they happen to meet.
 *      * Whoever uses this software for comercial purposes is morally obliged to buy 
 *        the author a 6pack of beer for every 1000 earned directly(including selling
 *        a hardware that is supported by this software ) from this software.
 *	
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *	NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef _DBGINT_H_
#define _DBGINT_H_

#include "def.h"

#define CONN( A,B) A##B
#define DBG_PORT_LETTER	B

#define DBG_PORT	CONN( PORT,B )
#define DBG_PIN		CONN( PIN,B )
#define DBG_DDR		CONN( DDR,B )

#define PIN_DATA	3
#define PIN_CLOCK	4
#define PIN_RESET	2

// definition of the I/O pin to drive the "VENDOR" LED
#if defined USBASP_HW
#define	STATUS_LED		(C,1)
#else
#define	STATUS_LED		(D,5)
#endif

#define DBG_MASK ((1<<PIN_DATA)|(1<<PIN_CLOCK)|(1<<PIN_RESET))
#define EXP2(A)	(1<<(A))

#define DBG_DIR_IN()		DBG_DDR &= ~EXP2(PIN_DATA)
#define DBG_DIR_OUT()		DBG_DDR |= EXP2(PIN_DATA)
#define DBG_CLOCK_HIGH()	DBG_PORT |= EXP2(PIN_CLOCK);
#define DBG_CLOCK_LOW()		DBG_PORT &= ~EXP2(PIN_CLOCK);

#define DBG_RESET(A)		{ if(A){ DBG_DDR |= EXP2( PIN_RESET); }else{ DBG_DDR &= ~EXP2( PIN_RESET ); } }
#define DBG_DATA(A)			{ if(A){ DBG_PORT |= EXP2( PIN_DATA); }else{ DBG_PORT &= ~EXP2( PIN_DATA ); } }
#define DBG_IN			(DBG_PIN & EXP2(PIN_DATA))

#define DIR_OUT 1
#define DIR_IN	0

#define NOP()		asm("nop")

void delay( unsigned char d );
void dbg_init();
unsigned char dbg_send_byte( unsigned char ch, unsigned char dir );
void dbg_command( unsigned char *pIn, unsigned char cInCnt, unsigned char *pOut, unsigned char cOutCnt );
void dbg_enter();
void dbg_led( unsigned char state );

#define VENDOR_LED	1
#define VENDOR_RESET 2
#define VENDOR_ENTER_DEBUG_MODE 3
#define VENDOR_DEBUG 4
#define VENDOR_DEBUG_CLOCK_DELAY 5
#define VENDOR_XDATA32 6
#define VENDOR_READCODE 7

enum eCommands{
		CHIP_ERASE	= 0x14,
		WR_CONFIG	= 0x1D,
		READ_STATUS	= 0x34,
		GET_CHIP_ID = 0x68,
		HALT		= 0x44,
		RESUME		= 0x4C,
		DEBUG_INSTR1= 0x55,
		DEBUG_INSTR2= 0x56,
		DEBUG_INSTR3= 0x57
	};
	

extern	byte_t	usb_setup ( byte_t data[8] );

static	byte_t		sck_period=1;	// SCK period in microseconds (1..250)

__attribute__((always_inline))
static	void	delay_6 ( void )
{
	asm volatile(
		"	mov	__tmp_reg__,%0	\n"
		"0:	rjmp	1f		\n"
		"1:	nop			\n"
		"	dec	__tmp_reg__	\n"
		"	brne	0b		\n"
		: : "r" (sck_period) );
}

#endif //_DBGINT_H_
